@@ -1,27 +1,26 @@
-# coding=utf-8
 """
 Download mode implementation.
 
 """
-from __future__ import division
+import mimetypes
 import os
 import re
 import sys
-import errno
-import mimetypes
 import threading
-from time import sleep, time
 from mailbox import Message
+from time import sleep, monotonic
+from typing import IO, Optional, Tuple
+from urllib.parse import urlsplit
 
-from httpie.output.streams import RawStream
-from httpie.models import HTTPResponse
-from httpie.utils import humanize_bytes
-from httpie.compat import urlsplit
+import requests
+
+from .models import HTTPResponse
+from .output.streams import RawStream
+from .utils import humanize_bytes
 
 
 PARTIAL_CONTENT = 206
 
-
 CLEAR_LINE = '\r\033[K'
 PROGRESS = (
     '{percentage: 6.2f} %'
@@ -38,11 +37,11 @@ class ContentRangeError(ValueError):
     pass
 
 
-def parse_content_range(content_range, resumed_from):
+def parse_content_range(content_range: str, resumed_from: int) -> int:
     """
     Parse and validate Content-Range header.
 
-    <http://www.w3.org/Protocols/rfc2616/rfc2616-sec14.html>
+    <https://www.w3.org/Protocols/rfc2616/rfc2616-sec14.html>
 
     :param content_range: the value of a Content-Range response header
                           eg. "bytes 21010-47021/47022"
@@ -61,7 +60,7 @@ def parse_content_range(content_range, resumed_from):
 
     if not match:
         raise ContentRangeError(
-            'Invalid Content-Range format %r' % content_range)
+            f'Invalid Content-Range format {content_range!r}')
 
     content_range_dict = match.groupdict()
     first_byte_pos = int(content_range_dict['first_byte_pos'])
@@ -78,26 +77,27 @@ def parse_content_range(content_range, resumed_from):
     # last-byte-pos value, is invalid. The recipient of an invalid
     # byte-content-range- spec MUST ignore it and any content
     # transferred along with it."
-    if (first_byte_pos >= last_byte_pos
-            or (instance_length is not None
-                and instance_length <= last_byte_pos)):
+    if (first_byte_pos > last_byte_pos
+        or (instance_length is not None
+            and instance_length <= last_byte_pos)):
         raise ContentRangeError(
-            'Invalid Content-Range returned: %r' % content_range)
+            f'Invalid Content-Range returned: {content_range!r}')
 
     if (first_byte_pos != resumed_from
-            or (instance_length is not None
-                and last_byte_pos + 1 != instance_length)):
+        or (instance_length is not None
+            and last_byte_pos + 1 != instance_length)):
         # Not what we asked for.
         raise ContentRangeError(
-            'Unexpected Content-Range returned (%r)'
-            ' for the requested Range ("bytes=%d-")'
-            % (content_range, resumed_from)
+            f'Unexpected Content-Range returned ({content_range!r})'
+            f' for the requested Range ("bytes={resumed_from}-")'
         )
 
     return last_byte_pos + 1
 
 
-def filename_from_content_disposition(content_disposition):
+def filename_from_content_disposition(
+    content_disposition: str
+) -> Optional[str]:
     """
     Extract and validate filename from a Content-Disposition header.
 
@@ -107,7 +107,7 @@ def filename_from_content_disposition(content_disposition):
     """
     # attachment; filename=jakubroztocil-httpie-0.4.1-20-g40bd8f6.tar.gz
 
-    msg = Message('Content-Disposition: %s' % content_disposition)
+    msg = Message(f'Content-Disposition: {content_disposition}')
     filename = msg.get_filename()
     if filename:
         # Basic sanitation.
@@ -116,7 +116,7 @@ def filename_from_content_disposition(content_disposition):
             return filename
 
 
-def filename_from_url(url, content_type):
+def filename_from_url(url: str, content_type: Optional[str]) -> str:
     fn = urlsplit(url).path.rstrip('/')
     fn = os.path.basename(fn) if fn else 'index'
     if '.' not in fn and content_type:
@@ -127,7 +127,7 @@ def filename_from_url(url, content_type):
         else:
             ext = mimetypes.guess_extension(content_type)
 
-        if ext == '.htm':  # Python 3
+        if ext == '.htm':
             ext = '.html'
 
         if ext:
@@ -136,7 +136,7 @@ def filename_from_url(url, content_type):
     return fn
 
 
-def trim_filename(filename, max_len):
+def trim_filename(filename: str, max_len: int) -> str:
     if len(filename) > max_len:
         trim_by = len(filename) - max_len
         name, ext = os.path.splitext(filename)
@@ -147,32 +147,24 @@ def trim_filename(filename, max_len):
     return filename
 
 
-def get_filename_max_length(directory):
+def get_filename_max_length(directory: str) -> int:
     max_len = 255
-    try:
-        pathconf = os.pathconf
-    except AttributeError:
-        pass  # non-posix
-    else:
-        try:
-            max_len = pathconf(directory, 'PC_NAME_MAX')
-        except OSError as e:
-            if e.errno != errno.EINVAL:
-                raise
+    if hasattr(os, 'pathconf') and 'PC_NAME_MAX' in os.pathconf_names:
+        max_len = os.pathconf(directory, 'PC_NAME_MAX')
     return max_len
 
 
-def trim_filename_if_needed(filename, directory='.', extra=0):
+def trim_filename_if_needed(filename: str, directory='.', extra=0) -> str:
     max_len = get_filename_max_length(directory) - extra
     if len(filename) > max_len:
         filename = trim_filename(filename, max_len)
     return filename
 
 
-def get_unique_filename(filename, exists=os.path.exists):
+def get_unique_filename(filename: str, exists=os.path.exists) -> str:
     attempt = 0
     while True:
-        suffix = '-' + str(attempt) if attempt > 0 else ''
+        suffix = f'-{attempt}' if attempt > 0 else ''
         try_filename = trim_filename_if_needed(filename, extra=len(suffix))
         try_filename += suffix
         if not exists(try_filename):
@@ -180,14 +172,17 @@ def get_unique_filename(filename, exists=os.path.exists):
         attempt += 1
 
 
-class Downloader(object):
+class Downloader:
 
-    def __init__(self, output_file=None,
-                 resume=False, progress_file=sys.stderr):
+    def __init__(
+        self,
+        output_file: IO = None,
+        resume: bool = False,
+        progress_file: IO = sys.stderr
+    ):
         """
         :param resume: Should the download resume if partial download
                        already exists.
-        :type resume: bool
 
         :param output_file: The file to store response body in. If not
                             provided, it will be guessed from the response.
@@ -195,24 +190,21 @@ class Downloader(object):
         :param progress_file: Where to report download progress.
 
         """
+        self.finished = False
+        self.status = DownloadStatus()
         self._output_file = output_file
         self._resume = resume
         self._resumed_from = 0
-        self.finished = False
-
-        self.status = Status()
         self._progress_reporter = ProgressReporterThread(
             status=self.status,
             output=progress_file
         )
 
-    def pre_request(self, request_headers):
+    def pre_request(self, request_headers: dict):
         """Called just before the HTTP request is sent.
 
         Might alter `request_headers`.
 
-        :type request_headers: dict
-
         """
         # Ask the server not to encode the content so that we can resume, etc.
         request_headers['Accept-Encoding'] = 'identity'
@@ -221,16 +213,20 @@ class Downloader(object):
             if bytes_have:
                 # Set ``Range`` header to resume the download
                 # TODO: Use "If-Range: mtime" to make sure it's fresh?
-                request_headers['Range'] = 'bytes=%d-' % bytes_have
+                request_headers['Range'] = f'bytes={bytes_have}-'
                 self._resumed_from = bytes_have
 
-    def start(self, final_response):
+    def start(
+        self,
+        initial_url: str,
+        final_response: requests.Response
+    ) -> Tuple[RawStream, IO]:
         """
         Initiate and return a stream for `response` body  with progress
         callback attached. Can be called only once.
 
+        :param initial_url: The original requested URL
         :param final_response: Initiated response object with headers already fetched
-        :type final_response: requests.models.Response
 
         :return: RawStream, output_file
 
@@ -238,7 +234,7 @@ class Downloader(object):
         assert not self.status.time_started
 
         # FIXME: some servers still might sent Content-Encoding: gzip
-        # <https://github.com/jakubroztocil/httpie/issues/423>
+        # <https://github.com/httpie/httpie/issues/423>
         try:
             total_size = int(final_response.headers['Content-Length'])
         except (KeyError, ValueError, TypeError):
@@ -246,7 +242,9 @@ class Downloader(object):
 
         if not self._output_file:
             self._output_file = self._get_output_file_from_response(
-                final_response)
+                initial_url=initial_url,
+                final_response=final_response,
+            )
         else:
             # `--output, -o` provided
             if self._resume and final_response.status_code == PARTIAL_CONTENT:
@@ -260,7 +258,7 @@ class Downloader(object):
                 try:
                     self._output_file.seek(0)
                     self._output_file.truncate()
-                except IOError:
+                except OSError:
                     pass  # stdout
 
         self.status.started(
@@ -277,12 +275,8 @@ class Downloader(object):
         )
 
         self._progress_reporter.output.write(
-            'Downloading %sto "%s"\n' % (
-                (humanize_bytes(total_size) + ' '
-                 if total_size is not None
-                 else ''),
-                self._output_file.name
-            )
+            f'Downloading {humanize_bytes(total_size) + " " if total_size is not None else ""}'
+            f'to "{self._output_file.name}"\n'
         )
         self._progress_reporter.start()
 
@@ -297,46 +291,44 @@ class Downloader(object):
         self._progress_reporter.stop()
 
     @property
-    def interrupted(self):
+    def interrupted(self) -> bool:
         return (
             self.finished
             and self.status.total_size
             and self.status.total_size != self.status.downloaded
         )
 
-    def chunk_downloaded(self, chunk):
+    def chunk_downloaded(self, chunk: bytes):
         """
         A download progress callback.
 
         :param chunk: A chunk of response body data that has just
                       been downloaded and written to the output.
-        :type chunk: bytes
 
         """
         self.status.chunk_downloaded(len(chunk))
 
     @staticmethod
-    def _get_output_file_from_response(final_response):
+    def _get_output_file_from_response(
+        initial_url: str,
+        final_response: requests.Response,
+    ) -> IO:
         # Output file not specified. Pick a name that doesn't exist yet.
         filename = None
         if 'Content-Disposition' in final_response.headers:
             filename = filename_from_content_disposition(
                 final_response.headers['Content-Disposition'])
         if not filename:
-            initial_response = (
-                final_response.history[0] if final_response.history
-                else final_response
-            )
             filename = filename_from_url(
-                url=initial_response.url,
+                url=initial_url,
                 content_type=final_response.headers.get('Content-Type'),
             )
         unique_filename = get_unique_filename(filename)
         return open(unique_filename, mode='a+b')
 
 
-class Status(object):
-    """Holds details about the downland status."""
+class DownloadStatus:
+    """Holds details about the download status."""
 
     def __init__(self):
         self.downloaded = 0
@@ -349,7 +341,7 @@ class Status(object):
         assert self.time_started is None
         self.total_size = total_size
         self.downloaded = self.resumed_from = resumed_from
-        self.time_started = time()
+        self.time_started = monotonic()
 
     def chunk_downloaded(self, size):
         assert self.time_finished is None
@@ -362,7 +354,7 @@ class Status(object):
     def finished(self):
         assert self.time_started is not None
         assert self.time_finished is None
-        self.time_finished = time()
+        self.time_finished = monotonic()
 
 
 class ProgressReporterThread(threading.Thread):
@@ -372,13 +364,15 @@ class ProgressReporterThread(threading.Thread):
     Uses threading to periodically update the status (speed, ETA, etc.).
 
     """
-    def __init__(self, status, output, tick=.1, update_interval=1):
-        """
 
-        :type status: Status
-        :type output: file
-        """
-        super(ProgressReporterThread, self).__init__()
+    def __init__(
+        self,
+        status: DownloadStatus,
+        output: IO,
+        tick=.1,
+        update_interval=1
+    ):
+        super().__init__()
         self.status = status
         self.output = output
         self._tick = tick
@@ -386,7 +380,7 @@ class ProgressReporterThread(threading.Thread):
         self._spinner_pos = 0
         self._status_line = ''
         self._prev_bytes = 0
-        self._prev_time = time()
+        self._prev_time = monotonic()
         self._should_stop = threading.Event()
 
     def stop(self):
@@ -403,16 +397,11 @@ class ProgressReporterThread(threading.Thread):
             sleep(self._tick)
 
     def report_speed(self):
-
-        now = time()
-
+        now = monotonic()
         if now - self._prev_time >= self._update_interval:
             downloaded = self.status.downloaded
-            try:
-                speed = ((downloaded - self._prev_bytes)
-                         / (now - self._prev_time))
-            except ZeroDivisionError:
-                speed = 0
+            speed = ((downloaded - self._prev_bytes)
+                     / (now - self._prev_time))
 
             if not self.status.total_size:
                 self._status_line = PROGRESS_NO_CONTENT_LENGTH.format(
@@ -420,10 +409,9 @@ class ProgressReporterThread(threading.Thread):
                     speed=humanize_bytes(speed),
                 )
             else:
-                try:
-                    percentage = downloaded / self.status.total_size * 100
-                except ZeroDivisionError:
-                    percentage = 0
+                percentage = (downloaded / self.status.total_size * 100
+                              if self.status.total_size
+                              else 0)
 
                 if not speed:
                     eta = '-:--:--'
@@ -431,7 +419,7 @@ class ProgressReporterThread(threading.Thread):
                     s = int((self.status.total_size - downloaded) / speed)
                     h, s = divmod(s, 60 * 60)
                     m, s = divmod(s, 60)
-                    eta = '{0}:{1:0>2}:{2:0>2}'.format(h, m, s)
+                    eta = f'{h}:{m:0>2}:{s:0>2}'
 
                 self._status_line = PROGRESS.format(
                     percentage=percentage,
@@ -444,33 +432,20 @@ class ProgressReporterThread(threading.Thread):
             self._prev_bytes = downloaded
 
         self.output.write(
-            CLEAR_LINE
-            + ' '
-            + SPINNER[self._spinner_pos]
-            + ' '
-            + self._status_line
+            f'{CLEAR_LINE} {SPINNER[self._spinner_pos]} {self._status_line}'
         )
         self.output.flush()
 
-        self._spinner_pos = (self._spinner_pos + 1
-                             if self._spinner_pos + 1 != len(SPINNER)
-                             else 0)
+        self._spinner_pos = (self._spinner_pos + 1) % len(SPINNER)
 
     def sum_up(self):
         actually_downloaded = (
             self.status.downloaded - self.status.resumed_from)
         time_taken = self.status.time_finished - self.status.time_started
+        speed = actually_downloaded / time_taken if time_taken else actually_downloaded
 
         self.output.write(CLEAR_LINE)
 
-        try:
-            speed = actually_downloaded / time_taken
-        except ZeroDivisionError:
-            # Either time is 0 (not all systems provide `time.time`
-            # with a better precision than 1 second), and/or nothing
-            # has been downloaded.
-            speed = actually_downloaded
-
         self.output.write(SUMMARY.format(
             downloaded=humanize_bytes(actually_downloaded),
             total=(self.status.total_size
