@@ -1,6 +1,9 @@
-from typing import Iterable, Optional
+from typing import Iterable
 from urllib.parse import urlsplit
 
+from .utils import split_cookies, parse_content_type_header
+from .compat import cached_property
+
 
 class HTTPMessage:
     """Abstract class for HTTP messages."""
@@ -10,33 +13,28 @@ class HTTPMessage:
 
     def iter_body(self, chunk_size: int) -> Iterable[bytes]:
         """Return an iterator over the body."""
-        raise NotImplementedError()
+        raise NotImplementedError
 
     def iter_lines(self, chunk_size: int) -> Iterable[bytes]:
         """Return an iterator over the body yielding (`line`, `line_feed`)."""
-        raise NotImplementedError()
+        raise NotImplementedError
 
     @property
     def headers(self) -> str:
         """Return a `str` with the message's headers."""
-        raise NotImplementedError()
+        raise NotImplementedError
 
-    @property
-    def encoding(self) -> Optional[str]:
-        """Return a `str` with the message's encoding, if known."""
-        raise NotImplementedError()
-
-    @property
-    def body(self) -> bytes:
-        """Return a `bytes` with the message's body."""
-        raise NotImplementedError()
+    @cached_property
+    def encoding(self) -> str:
+        ct, params = parse_content_type_header(self.content_type)
+        return params.get('charset', '')
 
     @property
     def content_type(self) -> str:
         """Return the message content type."""
         ct = self._orig.headers.get('Content-Type', '')
         if not isinstance(ct, str):
-            ct = ct.decode('utf8')
+            ct = ct.decode()
         return ct
 
 
@@ -52,39 +50,32 @@ class HTTPResponse(HTTPMessage):
     # noinspection PyProtectedMember
     @property
     def headers(self):
-        original = self._orig.raw._original_response
-
+        try:
+            raw_version = self._orig.raw._original_response.version
+        except AttributeError:
+            # Assume HTTP/1.1
+            raw_version = 11
         version = {
             9: '0.9',
             10: '1.0',
             11: '1.1',
             20: '2',
-        }[original.version]
+        }[raw_version]
 
-        status_line = f'HTTP/{version} {original.status} {original.reason}'
+        original = self._orig
+        status_line = f'HTTP/{version} {original.status_code} {original.reason}'
         headers = [status_line]
-        try:
-            # `original.msg` is a `http.client.HTTPMessage` on Python 3
-            # `_headers` is a 2-tuple
-            headers.extend(
-                '%s: %s' % header for header in original.msg._headers)
-        except AttributeError:
-            # and a `httplib.HTTPMessage` on Python 2.x
-            # `headers` is a list of `name: val<CRLF>`.
-            headers.extend(h.strip() for h in original.msg.headers)
-
+        headers.extend(
+            ': '.join(header)
+            for header in original.headers.items()
+            if header[0] != 'Set-Cookie'
+        )
+        headers.extend(
+            f'Set-Cookie: {cookie}'
+            for cookie in split_cookies(original.headers.get('Set-Cookie'))
+        )
         return '\r\n'.join(headers)
 
-    @property
-    def encoding(self):
-        return self._orig.encoding or 'utf8'
-
-    @property
-    def body(self):
-        # Only now the response body is fetched.
-        # Shouldn't be touched unless the body is actually needed.
-        return self._orig.content
-
 
 class HTTPRequest(HTTPMessage):
     """A :class:`requests.models.Request` wrapper."""
@@ -102,7 +93,7 @@ class HTTPRequest(HTTPMessage):
         request_line = '{method} {path}{query} HTTP/1.1'.format(
             method=self._orig.method,
             path=url.path or '/',
-            query='?' + url.query if url.query else ''
+            query=f'?{url.query}' if url.query else ''
         )
 
         headers = dict(self._orig.headers)
@@ -110,29 +101,18 @@ class HTTPRequest(HTTPMessage):
             headers['Host'] = url.netloc.split('@')[-1]
 
         headers = [
-            '%s: %s' % (
-                name,
-                value if isinstance(value, str) else value.decode('utf8')
-            )
+            f'{name}: {value if isinstance(value, str) else value.decode()}'
             for name, value in headers.items()
         ]
 
         headers.insert(0, request_line)
         headers = '\r\n'.join(headers).strip()
-
-        if isinstance(headers, bytes):
-            # Python < 3
-            headers = headers.decode('utf8')
         return headers
 
-    @property
-    def encoding(self):
-        return 'utf8'
-
     @property
     def body(self):
         body = self._orig.body
         if isinstance(body, str):
             # Happens with JSON/form request data parsed from the command line.
-            body = body.encode('utf8')
+            body = body.encode()
         return body or b''
