@@ -1,19 +1,23 @@
-from __future__ import absolute_import
 import json
+from typing import Optional, Type
 
 import pygments.lexer
-import pygments.token
-import pygments.styles
 import pygments.lexers
 import pygments.style
+import pygments.styles
+import pygments.token
 from pygments.formatters.terminal import TerminalFormatter
 from pygments.formatters.terminal256 import Terminal256Formatter
+from pygments.lexer import Lexer
+from pygments.lexers.data import JsonLexer
 from pygments.lexers.special import TextLexer
 from pygments.lexers.text import HttpLexer as PygmentsHttpLexer
 from pygments.util import ClassNotFound
 
-from httpie.compat import is_windows
-from httpie.plugins import FormatterPlugin
+from ..lexers.json import EnhancedJsonLexer
+from ...compat import is_windows
+from ...context import Environment
+from ...plugins import FormatterPlugin
 
 
 AUTO_STYLE = 'auto'  # Follows terminal ANSI color styles
@@ -24,7 +28,6 @@ if is_windows:
     # great and fruity seems to give the best result there.
     DEFAULT_STYLE = 'fruity'
 
-
 AVAILABLE_STYLES = set(pygments.styles.get_all_styles())
 AVAILABLE_STYLES.add(SOLARIZED_STYLE)
 AVAILABLE_STYLES.add(AUTO_STYLE)
@@ -40,9 +43,14 @@ class ColorFormatter(FormatterPlugin):
     """
     group_name = 'colors'
 
-    def __init__(self, env, explicit_json=False,
-                 color_scheme=DEFAULT_STYLE, **kwargs):
-        super(ColorFormatter, self).__init__(**kwargs)
+    def __init__(
+        self,
+        env: Environment,
+        explicit_json=False,
+        color_scheme=DEFAULT_STYLE,
+        **kwargs
+    ):
+        super().__init__(**kwargs)
 
         if not env.colors:
             self.enabled = False
@@ -54,6 +62,7 @@ class ColorFormatter(FormatterPlugin):
             http_lexer = PygmentsHttpLexer()
             formatter = TerminalFormatter()
         else:
+            from ..lexers.http import SimplifiedHTTPLexer
             http_lexer = SimplifiedHTTPLexer()
             formatter = Terminal256Formatter(
                 style=self.get_style_class(color_scheme)
@@ -63,14 +72,14 @@ class ColorFormatter(FormatterPlugin):
         self.formatter = formatter
         self.http_lexer = http_lexer
 
-    def format_headers(self, headers):
+    def format_headers(self, headers: str) -> str:
         return pygments.highlight(
             code=headers,
             lexer=self.http_lexer,
             formatter=self.formatter,
         ).strip()
 
-    def format_body(self, body, mime):
+    def format_body(self, body: str, mime: str) -> str:
         lexer = self.get_lexer_for_body(mime, body)
         if lexer:
             body = pygments.highlight(
@@ -78,24 +87,31 @@ class ColorFormatter(FormatterPlugin):
                 lexer=lexer,
                 formatter=self.formatter,
             )
-        return body.strip()
+        return body
 
-    def get_lexer_for_body(self, mime, body):
+    def get_lexer_for_body(
+        self, mime: str,
+        body: str
+    ) -> Optional[Type[Lexer]]:
         return get_lexer(
             mime=mime,
             explicit_json=self.explicit_json,
             body=body,
         )
 
-    def get_style_class(self, color_scheme):
+    @staticmethod
+    def get_style_class(color_scheme: str) -> Type[pygments.style.Style]:
         try:
             return pygments.styles.get_style_by_name(color_scheme)
         except ClassNotFound:
             return Solarized256Style
 
 
-def get_lexer(mime, explicit_json=False, body=''):
-
+def get_lexer(
+    mime: str,
+    explicit_json=False,
+    body=''
+) -> Optional[Type[Lexer]]:
     # Build candidate mime type and lexer names.
     mime_types, lexer_names = [mime], []
     type_, subtype = mime.split('/', 1)
@@ -105,8 +121,8 @@ def get_lexer(mime, explicit_json=False, body=''):
         subtype_name, subtype_suffix = subtype.split('+', 1)
         lexer_names.extend([subtype_name, subtype_suffix])
         mime_types.extend([
-            '%s/%s' % (type_, subtype_name),
-            '%s/%s' % (type_, subtype_suffix)
+            f'{type_}/{subtype_name}',
+            f'{type_}/{subtype_suffix}',
         ])
 
     # As a last resort, if no lexer feels responsible, and
@@ -138,57 +154,14 @@ def get_lexer(mime, explicit_json=False, body=''):
         else:
             lexer = pygments.lexers.get_lexer_by_name('json')
 
+    # Use our own JSON lexer: it supports JSON bodies preceded by non-JSON data
+    # as well as legit JSON bodies.
+    if isinstance(lexer, JsonLexer):
+        lexer = EnhancedJsonLexer()
+
     return lexer
 
 
-class SimplifiedHTTPLexer(pygments.lexer.RegexLexer):
-    """Simplified HTTP lexer for Pygments.
-
-    It only operates on headers and provides a stronger contrast between
-    their names and values than the original one bundled with Pygments
-    (:class:`pygments.lexers.text import HttpLexer`), especially when
-    Solarized color scheme is used.
-
-    """
-    name = 'HTTP'
-    aliases = ['http']
-    filenames = ['*.http']
-    tokens = {
-        'root': [
-            # Request-Line
-            (r'([A-Z]+)( +)([^ ]+)( +)(HTTP)(/)(\d+\.\d+)',
-             pygments.lexer.bygroups(
-                 pygments.token.Name.Function,
-                 pygments.token.Text,
-                 pygments.token.Name.Namespace,
-                 pygments.token.Text,
-                 pygments.token.Keyword.Reserved,
-                 pygments.token.Operator,
-                 pygments.token.Number
-             )),
-            # Response Status-Line
-            (r'(HTTP)(/)(\d+\.\d+)( +)(\d{3})( +)(.+)',
-             pygments.lexer.bygroups(
-                 pygments.token.Keyword.Reserved,  # 'HTTP'
-                 pygments.token.Operator,  # '/'
-                 pygments.token.Number,  # Version
-                 pygments.token.Text,
-                 pygments.token.Number,  # Status code
-                 pygments.token.Text,
-                 pygments.token.Name.Exception,  # Reason
-             )),
-            # Header
-            (r'(.*?)( *)(:)( *)(.+)', pygments.lexer.bygroups(
-                pygments.token.Name.Attribute,  # Name
-                pygments.token.Text,
-                pygments.token.Operator,  # Colon
-                pygments.token.Text,
-                pygments.token.String  # Value
-            ))
-        ]
-    }
-
-
 class Solarized256Style(pygments.style.Style):
     """
     solarized256
