@@ -1,9 +1,9 @@
 """Tests for dealing with binary request and response data."""
 import requests
 
-from fixtures import BIN_FILE_PATH, BIN_FILE_CONTENT, BIN_FILE_PATH_ARG
+from .fixtures import BIN_FILE_PATH, BIN_FILE_CONTENT, BIN_FILE_PATH_ARG
 from httpie.output.streams import BINARY_SUPPRESSED_NOTICE
-from utils import MockEnvironment, http
+from .utils import MockEnvironment, http
 
 
 class TestBinaryRequestData:
@@ -21,7 +21,7 @@ class TestBinaryRequestData:
     def test_binary_file_path(self, httpbin):
         env = MockEnvironment(stdin_isatty=True, stdout_isatty=False)
         r = http('--print=B', 'POST', httpbin.url + '/post',
-                 '@' + BIN_FILE_PATH_ARG, env=env, )
+                 '@' + BIN_FILE_PATH_ARG, env=env)
         assert r == BIN_FILE_CONTENT
 
     def test_binary_file_form(self, httpbin):
