@@ -1,10 +1,10 @@
-import mock
+from unittest import mock
 from pytest import raises
 from requests import Request
 from requests.exceptions import ConnectionError
 
 from httpie.status import ExitStatus
-from utils import HTTP_OK, http
+from .utils import HTTP_OK, http
 
 
 @mock.patch('httpie.core.program')
@@ -39,3 +39,21 @@ def test_max_headers_limit(httpbin_both):
 
 def test_max_headers_no_limit(httpbin_both):
     assert HTTP_OK in http('--max-headers=0', httpbin_both + '/get')
+
+
+def test_response_charset_option_unknown_encoding(httpbin_both):
+    r = http(
+        '--response-charset=foobar',
+        httpbin_both + '/get',
+        tolerate_error_exit_status=True,
+    )
+    assert "'foobar' is not a supported encoding" in r.stderr
+
+
+def test_response_mime_option_invalid_mime_type(httpbin_both):
+    r = http(
+        '--response-mime=foobar',
+        httpbin_both + '/get',
+        tolerate_error_exit_status=True,
+    )
+    assert "'foobar' doesn’t look like a mime type" in r.stderr
