@@ -1,37 +1,157 @@
 import argparse
+from pathlib import Path
+from unittest import mock
+
 import json
 import os
-from tempfile import gettempdir
+import io
 from urllib.request import urlopen
 
 import pytest
+import requests
+import responses
 
 from httpie.cli.argtypes import (
     PARSED_DEFAULT_FORMAT_OPTIONS,
     parse_format_options,
 )
 from httpie.cli.definition import parser
+from httpie.encoding import UTF8
 from httpie.output.formatters.colors import get_lexer
 from httpie.status import ExitStatus
-from utils import COLOR, CRLF, HTTP_OK, MockEnvironment, http
+from .fixtures import XML_DATA_RAW, XML_DATA_FORMATTED
+from .utils import COLOR, CRLF, HTTP_OK, MockEnvironment, http, DUMMY_URL
 
 
 @pytest.mark.parametrize('stdout_isatty', [True, False])
-def test_output_option(httpbin, stdout_isatty):
-    output_filename = os.path.join(gettempdir(), test_output_option.__name__)
+def test_output_option(tmp_path, httpbin, stdout_isatty):
+    output_filename = tmp_path / 'test_output_option'
     url = httpbin + '/robots.txt'
 
-    r = http('--output', output_filename, url,
+    r = http('--output', str(output_filename), url,
              env=MockEnvironment(stdout_isatty=stdout_isatty))
     assert r == ''
 
     expected_body = urlopen(url).read().decode()
-    with open(output_filename, 'r') as f:
-        actual_body = f.read()
+    actual_body = output_filename.read_text(encoding=UTF8)
 
     assert actual_body == expected_body
 
 
+class TestQuietFlag:
+    QUIET_SCENARIOS = [('--quiet',), ('-q',), ('--quiet', '--quiet'), ('-qq',)]
+
+    @pytest.mark.parametrize('quiet_flags', QUIET_SCENARIOS)
+    def test_quiet(self, httpbin, quiet_flags):
+        env = MockEnvironment(
+            stdin_isatty=True,
+            stdout_isatty=True,
+            devnull=io.BytesIO()
+        )
+        r = http(*quiet_flags, 'GET', httpbin.url + '/get', env=env)
+        assert env.stdout is env.devnull
+        assert env.stderr is env.devnull
+        assert HTTP_OK in r.devnull
+        assert r == ''
+        assert r.stderr == ''
+
+    def test_quiet_with_check_status_non_zero(self, httpbin):
+        r = http(
+            '--quiet', '--check-status', httpbin + '/status/500',
+            tolerate_error_exit_status=True,
+        )
+        assert 'http: warning: HTTP 500' in r.stderr
+
+    def test_quiet_with_check_status_non_zero_pipe(self, httpbin):
+        r = http(
+            '--quiet', '--check-status', httpbin + '/status/500',
+            tolerate_error_exit_status=True,
+            env=MockEnvironment(stdout_isatty=False)
+        )
+        assert 'http: warning: HTTP 500' in r.stderr
+
+    def test_quiet_quiet_with_check_status_non_zero(self, httpbin):
+        r = http(
+            '--quiet', '--quiet', '--check-status', httpbin + '/status/500',
+            tolerate_error_exit_status=True,
+        )
+        assert not r.stderr
+
+    def test_quiet_quiet_with_check_status_non_zero_pipe(self, httpbin):
+        r = http(
+            '--quiet', '--quiet', '--check-status', httpbin + '/status/500',
+            tolerate_error_exit_status=True,
+            env=MockEnvironment(stdout_isatty=False)
+        )
+        assert 'http: warning: HTTP 500' in r.stderr
+
+    @pytest.mark.parametrize('quiet_flags', QUIET_SCENARIOS)
+    @mock.patch('httpie.cli.argtypes.AuthCredentials._getpass',
+                new=lambda self, prompt: 'password')
+    def test_quiet_with_password_prompt(self, httpbin, quiet_flags):
+        """
+        Tests whether httpie still prompts for a password when request
+        requires authentication and only username is provided
+
+        """
+        env = MockEnvironment(
+            stdin_isatty=True,
+            stdout_isatty=True,
+            devnull=io.BytesIO()
+        )
+        r = http(
+            *quiet_flags, '--auth', 'user', 'GET',
+            httpbin.url + '/basic-auth/user/password',
+            env=env
+        )
+        assert env.stdout is env.devnull
+        assert env.stderr is env.devnull
+        assert HTTP_OK in r.devnull
+        assert r == ''
+        assert r.stderr == ''
+
+    @pytest.mark.parametrize('quiet_flags', QUIET_SCENARIOS)
+    @pytest.mark.parametrize('output_options', ['-h', '-b', '-v', '-p=hH'])
+    def test_quiet_with_explicit_output_options(self, httpbin, quiet_flags, output_options):
+        env = MockEnvironment(stdin_isatty=True, stdout_isatty=True)
+        r = http(*quiet_flags, output_options, httpbin.url + '/get', env=env)
+        assert env.stdout is env.devnull
+        assert env.stderr is env.devnull
+        assert r == ''
+        assert r.stderr == ''
+
+    @pytest.mark.parametrize('quiet_flags', QUIET_SCENARIOS)
+    @pytest.mark.parametrize('with_download', [True, False])
+    def test_quiet_with_output_redirection(self, tmp_path, httpbin, quiet_flags, with_download):
+        url = httpbin + '/robots.txt'
+        output_path = Path('output.txt')
+        env = MockEnvironment()
+        orig_cwd = os.getcwd()
+        output = requests.get(url).text
+        extra_args = ['--download'] if with_download else []
+        os.chdir(tmp_path)
+        try:
+            assert os.listdir('.') == []
+            r = http(
+                *quiet_flags,
+                '--output', str(output_path),
+                *extra_args,
+                url,
+                env=env
+            )
+            assert os.listdir('.') == [str(output_path)]
+            assert r == ''
+            assert r.stderr == ''
+            assert env.stderr is env.devnull
+            if with_download:
+                assert env.stdout is env.devnull
+            else:
+                assert env.stdout is not env.devnull  # --output swaps stdout.
+            assert output_path.read_text(encoding=UTF8) == output
+        finally:
+            os.chdir(orig_cwd)
+
+
 class TestVerboseFlag:
     def test_verbose(self, httpbin):
         r = http('--verbose',
@@ -39,8 +159,14 @@ class TestVerboseFlag:
         assert HTTP_OK in r
         assert r.count('__test__') == 2
 
+    def test_verbose_raw(self, httpbin):
+        r = http('--verbose', '--raw', 'foo bar',
+                 'POST', httpbin.url + '/post')
+        assert HTTP_OK in r
+        assert 'foo bar' in r
+
     def test_verbose_form(self, httpbin):
-        # https://github.com/jakubroztocil/httpie/issues/53
+        # https://github.com/httpie/httpie/issues/53
         r = http('--verbose', '--form', 'POST', httpbin.url + '/post',
                  'A=B', 'C=D')
         assert HTTP_OK in r
@@ -63,8 +189,8 @@ class TestVerboseFlag:
 class TestColors:
 
     @pytest.mark.parametrize(
-        argnames=['mime', 'explicit_json', 'body', 'expected_lexer_name'],
-        argvalues=[
+        'mime, explicit_json, body, expected_lexer_name',
+        [
             ('application/json', False, None, 'JSON'),
             ('application/json+foo', False, None, 'JSON'),
             ('application/foo+json', False, None, 'JSON'),
@@ -103,7 +229,7 @@ class TestPrettyOptions:
 
     def test_force_pretty(self, httpbin):
         env = MockEnvironment(stdout_isatty=False, colors=256)
-        r = http('--pretty=all', 'GET', httpbin.url + '/get', env=env, )
+        r = http('--pretty=all', 'GET', httpbin.url + '/get', env=env)
         assert COLOR in r
 
     def test_force_ugly(self, httpbin):
@@ -153,7 +279,7 @@ class TestLineEndings:
                 break
             assert header.endswith(CRLF), repr(header)
         else:
-            assert 0, 'CRLF between headers and body not found in %r' % msg
+            assert 0, f'CRLF between headers and body not found in {msg!r}'
         body = ''.join(lines)
         assert CRLF not in body
         return body
@@ -161,7 +287,7 @@ class TestLineEndings:
     def test_CRLF_headers_only(self, httpbin):
         r = http('--headers', 'GET', httpbin.url + '/get')
         body = self._validate_crlf(r)
-        assert not body, 'Garbage after headers: %r' % r
+        assert not body, f'Garbage after headers: {r!r}'
 
     def test_CRLF_ugly_response(self, httpbin):
         r = http('--pretty=none', 'GET', httpbin.url + '/get')
@@ -197,8 +323,8 @@ class TestFormatOptions:
         assert f'ZZZ: foo{CRLF}XXX: foo' in r_unsorted
 
     @pytest.mark.parametrize(
-        argnames=['options', 'expected_json'],
-        argvalues=[
+        'options, expected_json',
+        [
             # @formatter:off
             (
                 'json.sort_keys:true,json.indent:4',
@@ -224,8 +350,8 @@ class TestFormatOptions:
         assert expected_json in r
 
     @pytest.mark.parametrize(
-        argnames=['defaults', 'options_string', 'expected'],
-        argvalues=[
+        'defaults, options_string, expected',
+        [
             # @formatter:off
             ({'foo': {'bar': 1}}, 'foo.bar:2', {'foo': {'bar': 2}}),
             ({'foo': {'bar': True}}, 'foo.bar:false', {'foo': {'bar': False}}),
@@ -238,8 +364,8 @@ class TestFormatOptions:
         assert expected == actual
 
     @pytest.mark.parametrize(
-        argnames=['options_string', 'expected_error'],
-        argvalues=[
+        'options_string, expected_error',
+        [
             ('foo:2', 'invalid option'),
             ('foo.baz:2', 'invalid key'),
             ('foo.bar:false', 'expected int got bool'),
@@ -255,8 +381,8 @@ class TestFormatOptions:
             parse_format_options(s=options_string, defaults=defaults)
 
     @pytest.mark.parametrize(
-        argnames=['args', 'expected_format_options'],
-        argvalues=[
+        'args, expected_format_options',
+        [
             (
                 [
                     '--format-options',
@@ -272,6 +398,10 @@ class TestFormatOptions:
                         'indent': 10,
                         'format': True
                     },
+                    'xml': {
+                        'format': True,
+                        'indent': 2,
+                    },
                 }
             ),
             (
@@ -287,6 +417,10 @@ class TestFormatOptions:
                         'indent': 4,
                         'format': True
                     },
+                    'xml': {
+                        'format': True,
+                        'indent': 2,
+                    },
                 }
             ),
             (
@@ -304,6 +438,10 @@ class TestFormatOptions:
                         'indent': 4,
                         'format': True
                     },
+                    'xml': {
+                        'format': True,
+                        'indent': 2,
+                    },
                 }
             ),
             (
@@ -318,6 +456,8 @@ class TestFormatOptions:
             (
                 [
                     '--format-options=json.indent:2',
+                    '--format-options=xml.format:false',
+                    '--format-options=xml.indent:4',
                     '--unsorted',
                     '--no-unsorted',
                 ],
@@ -330,6 +470,10 @@ class TestFormatOptions:
                         'indent': 2,
                         'format': True
                     },
+                    'xml': {
+                        'format': False,
+                        'indent': 4,
+                    },
                 }
             ),
             (
@@ -347,6 +491,10 @@ class TestFormatOptions:
                         'indent': 2,
                         'format': True
                     },
+                    'xml': {
+                        'format': True,
+                        'indent': 2,
+                    },
                 }
             ),
             (
@@ -365,6 +513,10 @@ class TestFormatOptions:
                         'indent': 2,
                         'format': True
                     },
+                    'xml': {
+                        'format': True,
+                        'indent': 2,
+                    },
                 }
             ),
         ],
@@ -375,3 +527,35 @@ class TestFormatOptions:
             env=MockEnvironment(),
         )
         assert parsed_args.format_options == expected_format_options
+
+
+@responses.activate
+def test_response_mime_overwrite():
+    responses.add(
+        method=responses.GET,
+        url=DUMMY_URL,
+        body=XML_DATA_RAW,
+        content_type='text/plain',
+    )
+    r = http(
+        '--offline',
+        '--raw', XML_DATA_RAW,
+        '--response-mime=application/xml', DUMMY_URL
+    )
+    assert XML_DATA_RAW in r  # not affecting request bodies
+
+    r = http('--response-mime=application/xml', DUMMY_URL)
+    assert XML_DATA_FORMATTED in r
+
+
+@responses.activate
+def test_response_mime_overwrite_incorrect():
+    responses.add(
+        method=responses.GET,
+        url=DUMMY_URL,
+        body=XML_DATA_RAW,
+        content_type='text/xml',
+    )
+    # The provided Content-Type is simply ignored, and so no formatting is done.
+    r = http('--response-mime=incorrect/type', DUMMY_URL)
+    assert XML_DATA_RAW in r
